/*
 * Copyright (c) 1990, 1991, 1992 Stanford University
 *
 * Permission to use, copy, modify, and distribute this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name
 * Stanford may not be used in any advertising or publicity relating to
 * the software without the specific, prior written permission of
 * Stanford.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#include <multimedia/audio_hdr.h>
#include <desktop/tt_c.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/dragdrop.h>
#include <string.h>
#include <stdlib.h>
#include <sys/param.h>
#include "Video.h"
#include "player.h"
#include "jpeg.h"
#include <Sender.h>
#include <Receiver.h>

extern Attr_attribute	INSTANCE;
extern Player *player;
extern VideoIn   video;
extern Xv_Window frame;
extern Display *dpy;
extern int repeat_flag;
extern int silent_flag;
extern int abstract_flag;
extern int hackct;
extern Tt_message reply_msg;
extern int cur_width;
extern int cur_height;
extern int cur_itemsize;

extern Panel_item cntlitem;
extern Panel_item cntliteml;
extern Panel_item cntlitemr;
extern Panel_item fname;
extern int row_num;
extern int end_of_file;
extern Selection_requestor sel;
extern char		globalFilename[MAXPATHLEN];
extern char*		realFilename;

void 
  OpenDoc(char **filename)
{
  char *c;
  
  if (player)
    PlayerEject(player);
  
  player = (Player *) InitializeVideo(*filename, JPEG_FILE, video);
  if (player == NULL)
  {
    ShowAlert("Unable to initialize movie source","Open Failed",NULL);
    return;
  }
  
  if (xv_get(frame, FRAME_CLOSED))
    xv_set(frame, FRAME_CLOSED, FALSE, NULL);
  
  c = (char *) strrchr(*filename, 0x2f);
  if (c)
    c++;
  else
    c = *filename;
  xv_set(frame, FRAME_LEFT_FOOTER, c, NULL);
  
  AdjustFrame(*filename);
  
  if (silent_flag)
    SetPlayerAudio(player, AUDIO_OFF);
  
  if (strcmp(xv_get(frame, FRAME_RIGHT_FOOTER), "Playing"))
    xv_set(frame, FRAME_RIGHT_FOOTER, "abstract", NULL);
  strcpy(globalFilename, *filename);
}								    /* end function OpenDoc */


void 
  SetSelection(MAESelection *selection)				    /* At present, only complete movies can be performed;... */
{								    /* ...therefore, there is no need for SetSelection to... */
  return;							    /* ...do anything. */
}



void 
  PerformSelection(void)
{
  if (xv_get(frame, FRAME_CLOSED))
    xv_set(frame, FRAME_CLOSED, FALSE, NULL);
  
  if (strcmp(xv_get(frame, FRAME_RIGHT_FOOTER), "Playing"))
    xv_set(frame, FRAME_RIGHT_FOOTER, "Playing", NULL);
  
  PlayerPlay(player);
}								    /* end function PerformSelection */



char **
  GetDoc(void *unusedArg)
{
  if ((globalFilename == (char*)NULL) ||
      (realpath(globalFilename, realFilename) == NULL))
    strcpy(realFilename, "untitled");
  return (&realFilename);
}								    /* end function GetDoc */



MAESelection *
  GetSelection(void *unusedArg)
{
  static MAESelection select;
  jpheader *hdr;
  static int		firstTime = 0;
  
  if (firstTime == 0)						    /* Does space for a label need to be allocated? */
  {								    /* Yes, allocate and initialize that space */
/*    select.label = malloc(LabelLength);
      memset(select.label, 0, LabelLength);
*/
    firstTime = 1;
  }
  if (strcmp(globalFilename, "untitled") != 0)
  {
    hdr = (jpheader *) jpeg_movie_header(dpy, globalFilename);
    select.duration = (hdr->frames * 1000) / 30;
  }
  else
    select.duration = 0;
  
  select.start = 0;
  select.end = 0;
  select.offset = 0;
  sprintf(select.label, "No Label");
  return (&select);
}								    /* end function GetSelection */



void 
  HaltSelection(void)
{
  PlayerStop(player);
  return;
}

void 
  PauseSelection(void)
{
  PlayerStill(player);
  return;
}

void 
  ResumeSelection(void)
{
  PlayerPlay(player);
  return;
}


void 
  HideApplication(void)
{
  xv_set(frame, FRAME_CLOSED, TRUE, NULL);
  XFlush(dpy);
}

void 
  ShowApplication(void)
{
  xv_set(frame, FRAME_CLOSED, FALSE, NULL); 
  XRaiseWindow(dpy, (Window)xv_get(frame, XV_XID));
  XFlush(dpy);
}								    /* end function ShowApplication */


IconData *
  GetAppIcon(void)
{
  static int first = True;
  static IconData  iconData;
  static unsigned short baseWindow_bits[] = {
#include "icon/movietool.icon"
  };
  if (first)
    {
      iconData.iconData = (char *)malloc(sizeof(baseWindow_bits));
      memcpy(iconData.iconData, baseWindow_bits, sizeof(baseWindow_bits));
      if (iconData.iconData)
	iconData.dataLength = sizeof(baseWindow_bits);
      first = False;
    }
  return(&iconData);
}								    /* end function GetAppIcon */
