/***************************************************************************
 *
 *	audio.c - Audio digitialization part of vaa
 *
 *	(C) Copyright 1990,1991,1992 by Paradise Software, Inc.
 *
 *	Written by John Malleo-Roach
 *
 *	Version %G% - %I%
 *
 ***************************************************************************/

/* INCLUDE FILES */

#include	<stdio.h>
#include	<math.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<stropts.h>
#include	<sys/ioctl.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/param.h>
#include	<unistd.h>

#include	"vaa.h"

/* AUDIO STATIC VARIABLES */

static u_char AudioBuf[16384];		/* Buffer for storing input data				*/
static int AudioFile;				/* File descripter for output data				*/
static int AudioBytes;				/* Number of bytes read during this open		*/
static int AudioRequested;			/* Number of bytes requested for storage		*/
static int UpdateState;

#ifdef AUDIO
static Audio_hdr RecordHeader;		/* Audio record header for device				*/
#endif

/* AUDIO GLOBAL VARIABLES */

int    AudioFd=0;					/* File descriptor for audio device				*/
Window AudioXID;					/* X Window associated with AudioCanvas pw		*/
GC     AudioGCB;					/* Graphics context with Black foreground		*/
GC     AudioGCR;					/* Graphics context	with Red foreground			*/
GC     AudioGCW;					/* Graphics context	with White foreground		*/
int    AudioWidth;					/* Width of audio volume canvas					*/
int    AudioHeight;					/* Height of audio volume canvas				*/
int    AudioLevel;					/* Current remembered audio level				*/

/* FUNCTIONS */

/* Open up the audio device for playing and recording */

#ifdef AUDIO

/* Async audio SIGPOLL handler */

Notify_value SigPollHandler(client,sig,when)
Notify_client client;
int sig;
Notify_signal_mode when;
{
	int save,size,i;

	save = errno;

	size = 0;
	if (ioctl(AudioFd,FIONREAD,&size) < 0) {
		SystemError(errno,"FIONREAD ioctl failed");
		return;
	}
	if (size < 16) return;
	if ((i = read(AudioFd,AudioBuf,size)) < 0) {
		if (errno != EWOULDBLOCK) {
			SystemError(errno,"Audio read failed");
			return;
		}
	}
	if (AudioFile) {
		if ((AudioBytes + i) > AudioRequested) {
			i = AudioRequested - AudioBytes;
			if (i) {
				write(AudioFile,AudioBuf,i);
				AudioBytes += i;
			}
		}
		else {
			write(AudioFile,AudioBuf,i);
			AudioBytes += i;
		}
	}
	else {
/* Update state of device if changed
		if (!UpdateState) {
			UpdateState = TRUE;
			(void) notify_post_event(SIGPOLL, NULL, NOTIFY_SAFE);
		}
*/
	}
	errno = save;
	return(NOTIFY_DONE);
}

Notify_value UpdateHandler(client, event, arg, when)
Notify_client       client;
Notify_event        event;
Notify_arg          arg;
Notify_event_type   when;
{
	int value,flag;
	double gain,old;

	UpdateState = FALSE;

	audio_get_record_gain(AudioFd,&gain);
	value = (int) xv_get(RecordVolume,PANEL_VALUE);
	old = ((double) value) / ((double) MAX_GAIN);
	if (old != gain) audio_set_record_gain(AudioFd,&gain);

	audio_get_monitor_gain(AudioFd,&gain);
	value = (int) xv_get(MonitorVolume,PANEL_VALUE);
	gain = ((double) value) / ((double) MAX_GAIN);
	if (old != gain) audio_set_monitor_gain(AudioFd,&gain);

	audio_get_play_port(AudioFd,&flag);
	value = (int) xv_get(AudioOut,PANEL_VALUE);
	value = (value == 0 ? AUDIO_SPEAKER : AUDIO_HEADPHONE);
	if (value != flag) audio_set_play_port(AudioFd,&flag);
}
	
int OpenAudio(frames,pollit)
int frames;
int pollit;
{
	struct stat info;
	int flag,value;
	double gain;

	if ((AudioFd = open("/dev/audio",O_RDONLY|O_NDELAY)) < 0) {
		if ((errno == EBUSY) || (errno == EINTR)) {
			ShowNote("Audio device is already open by another process",NULL);
		}
		else SystemError(errno,"Could not open audio device");
		return(FALSE);
	}

/* Set up the audio device for asynchronous, non-blocking reads */

	flag = fcntl(AudioFd, F_GETFL, 0) | FNDELAY;
    if (fcntl(AudioFd, F_SETFL, flag) < 0) {
		SystemError(errno,"F_SETFL fcntl failed");
		close(AudioFd);
		return(FALSE);
	}
	if (pollit) {
		if (ioctl(AudioFd,I_SETSIG,S_INPUT|S_OUTPUT|S_MSG) < 0) {
			SystemError(errno,"I_SETSIG ioctl failed");
			close(AudioFd);
			return(FALSE);
		}
		(void) notify_set_signal_func(VAAFrame, SigPollHandler,
				  SIGPOLL, NOTIFY_ASYNC);
/*
		(void) notify_set_event_func(SIGPOLL, UpdateHandler, NOTIFY_SAFE);
*/
		UpdateState = FALSE;
	}

/* Get encoding information from device */

	if (audio_get_record_config(AudioFd,&RecordHeader) != AUDIO_SUCCESS) {
		ShowAlert("Cound not get record encoding configuration",NULL);
		return(FALSE);
	}

/* Set default record and monitor values */
/*
	gain = (double) 0.0; audio_set_play_gain(AudioFd,&gain);
*/
	value = (int) xv_get(RecordVolume,PANEL_VALUE);
	gain = ((double) value) / ((double) MAX_GAIN);
	audio_set_record_gain(AudioFd,&gain);
	value = (int) xv_get(MonitorVolume,PANEL_VALUE);
	gain = ((double) value) / ((double) MAX_GAIN);
	audio_set_monitor_gain(AudioFd,&gain);
	value = (int) xv_get(AudioOut,PANEL_VALUE);
	flag = (value == 0 ? AUDIO_SPEAKER : AUDIO_HEADPHONE);
	(void) audio_set_play_port(AudioFd,&flag);

	if (frames) {
		AudioFile = open("AUDIO",O_RDWR|O_CREAT,0666);
		if (AudioFile < 0) {
			ShowAlert("Could not create output audio file",NULL);
			AudioFile = 0; return(FALSE);
		}
		fstat(AudioFile,&info);
		if (info.st_size == 0) {
			if (audio_write_filehdr(AudioFile,&RecordHeader,NULL,NULL) != AUDIO_SUCCESS) {
				ShowAlert("Could not write audio file header",NULL);
				close(AudioFile); AudioFile = 0; return(FALSE);
			}
			RecordHeader.data_size = 0;
		}
		else {
			if (audio_read_filehdr(AudioFile,&RecordHeader,NULL,NULL) != AUDIO_SUCCESS) {
				ShowAlert("Could not read audio file header",NULL);
				close(AudioFile); AudioFile = 0; return(FALSE);
			}
			if (lseek(AudioFile,info.st_size, SEEK_SET) < 0) {
				SystemError(errno,"Can find end of audio file");
				close(AudioFile); AudioFile = 0; return(FALSE);
			}
		}
		value = (int) xv_get(FrameSpeed,PANEL_VALUE);
		AudioRequested = 8000 / value;
		if (8000 % value) AudioRequested++;
		AudioRequested = frames * AudioRequested;
	}
	else AudioFile = 0;
	AudioBytes = 0;
	return(TRUE);
}

int CloseAudio(frames,pollit)
int frames;
int pollit;
{
	int err;

	audio_pause_record(AudioFd);

	if (frames) {
		audio_get_record_error(AudioFd,(unsigned *)&err);
		if (err) ShowAlert("Data overflow occured",NULL);
		AudioBytes += RecordHeader.data_size;
		audio_rewrite_filesize(AudioFile,AudioBytes);
		AudioBytes     = 0;
		AudioRequested = 0;
	}
	else {
		audio_flush_record(AudioFd);
	}
	if (pollit) {
		(void) notify_set_signal_func(VAAFrame, NOTIFY_FUNC_NULL,
			  SIGPOLL, NOTIFY_ASYNC);
/*
		(void) notify_set_event_func(SIGPOLL, NOTIFY_FUNC_NULL,
			  NOTIFY_SAFE);
*/
	}
	close(AudioFd); AudioFd = 0;
}

/* Return TRUE when requested audio is satisfied */

BOOLEAN AudioDone()
{
	if (AudioBytes == AudioRequested) return(TRUE);
	else return(FALSE);
}

/* Draw our floating bar volume display */

void UpdateAudioDisplay()
{
	int width,height;
	int i,j,value;
	int x,y;

	value = 0;
	for (i=0; i<1024; i=i+64) {
		j = (int) AudioBuf[0];
		if (j & 128) j = 255 - j;
		else j = 128 - j;
		value += j;
	}
	value = value / 116;
	AudioLevel += (value - AudioLevel) / 2;
		
	XFillRectangle(VAADisplay,AudioXID,AudioGCB,20,0,AudioWidth,AudioHeight);

	width  = AudioWidth - 40;
	y      = AudioHeight / 16;
	height = AudioHeight / 18;

	if (AudioLevel < 9) {
		for (i=0; i<AudioLevel; i++) {
			XFillRectangle(VAADisplay,AudioXID,AudioGCW,30,AudioHeight-y*i,width,height);
		}
	}
	else {
		for (i=0; i<11; i++) {
			XFillRectangle(VAADisplay,AudioXID,AudioGCW,30,AudioHeight-y*i,width,height);
		}
		for (; i<AudioLevel; i++) {
			XFillRectangle(VAADisplay,AudioXID,AudioGCR,30,AudioHeight-y*i,width,height);
		}
	}
}

/* Panel notify function for the audio panel */

void AudioPanelProc(item, value, event)
Panel_item item;
int value;
Event *event;
{
	int who;
	unsigned port;
	double gain;

	who		= (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	
	switch (who) {
	case 1:
		port = (value == 0 ? AUDIO_SPEAKER : AUDIO_HEADPHONE);
		(void) audio_set_play_port(AudioFd,&port);
		break;
	case 2:
		gain = ((double) value) / ((double) MAX_GAIN);
		audio_set_record_gain(AudioFd,&gain);
		break;
	case 3:
		gain = ((double) value) / ((double) MAX_GAIN);
		audio_set_monitor_gain(AudioFd,&gain);
		break;
	}
}

/* Repaint proc for audio canvas */

void AudioRepaintProc(canvas,pw,rects)
Canvas canvas;
Xv_Window pw;
Rectlist *rects;
{
	int step,start;

	XFillRectangle(VAADisplay,AudioXID,AudioGCB,0,0,AudioWidth,AudioHeight);

	step  = AudioHeight / 11;
	start = AudioHeight - 5;

	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"DB",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"40",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"30",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"20",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start," 5",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start," 2",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start," 0",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start," 2",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start," 5",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start,"10",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start," +",2);
}

#else

int OpenAudio()
{
	;
}

Notify_value SigPollHandler()
{
	;
}

int CloseAudio()
{
	;
}

BOOLEAN AudioDone()
{
	;
}

void UpdateAudioDisplay()
{
	;
}

void AudioPanelProc()
{
	;
}

void AudioRepaintProc()
{
	;
}

#endif

/* end of audio.c */
