/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:	dand.c,v $
 * Revision 1.9  93/03/15  16:59:56  schen
 * new release from Paradise Software.
 * 
 * Revision 1.5  91/08/19  16:13:30  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: dand.c,v 1.9 93/03/15 16:59:56 schen Exp $ Parallax Graphics Inc";
#endif

/*
 *
 *	dand.c : OpenWindows interface for drag and drop video clips
 *		aka. videoclip
 *
 *	Written by John Malleo-Roach
 *
 */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/param.h>

#include	"Video.h"
#include	"player.h"

#include    <xview/seln.h>
#include    <xview/notify.h>
#include	<xview/textsw.h>

/* CONSTANTS */

#define PANEL_DATA		1001

#define icon_width 64
#define icon_height 64

static short icon_bits[] = {
#include "videoclip.icon"
};

static short mask_bits[] = {
#include "videoclip.mask"
};

#define BITS_WIDTH		32
#define BITS_HEIGHT		32

/* Define button glyphs for player control */

static short fastreverse_bits[] = {
#include "fastreverse.icon"
};
static short reverse_bits[] = {
#include "reverse.icon"
};
static short slowreverse_bits[] = {
#include "slowreverse.icon"
};
static short stepreverse_bits[] = {
#include "stepreverse.icon"
};
static short still_bits[] = {
#include "still.icon"
};
static short stepforward_bits[] = {
#include "stepforward.icon"
};
static short slowforward_bits[] = {
#include "slowforward.icon"
};
static short forward_bits[] = {
#include "forward.icon"
};
static short fastforward_bits[] = {
#include "fastforward.icon"
};
static short playsegment_bits[] = {
#include "playsegment.icon"
};

/* STATIC VARIABLES */

static char version[] = "@(#)$Id: dand.c,v 1.9 93/03/15 16:59:56 schen Exp $";

static VideoIn VideoCanvas;			/* Canvas pointer for video input			*/
static Panel   VideoPanel;			/* Panel pointer for user controls			*/
static Player *VideoPlayer;			/* Pointer to player type					*/
static Textsw  VideoText;			/* Area for notes							*/

static char filename[MAXPATHLEN];	/* Name of file dropped on our window		*/
static int  StartFrame;				/* Starting frame to show					*/
static int  EndFrame;				/* Ending frame to show						*/
static char Title[MAXPATHLEN];		/* Title of this clip						*/
static char Buffer[16384];

static int debug=FALSE;				/* Turn on debugging info					*/
static int savepid=FALSE;			/* Save pid for remote control				*/

static int PlayerModel;				/* Player specifics							*/
static int PlayerSpeed;
static char *PlayerDevice;

/* GLOBAL VARIABLES */

Frame   VAAFrame;					/* Frame pointer for base frame				*/

/* EXTERNAL DEFINITIONS */

extern int optind;
extern char *optarg;

/* FUNCTIONS */

/* Called on user exit from tool */

static Notify_error destroy_interposer(client,status)
Notify_client client;
Destroy_status status;
{
	if (status == DESTROY_CHECKING) ;
	else if (status == DESTROY_CLEANUP) {
		xv_destroy(VideoCanvas);
		return(notify_next_destroy_func(client,status));
	}
	else if (status == DESTROY_SAVE_YOURSELF) {
		xv_destroy(VideoCanvas);
	}
	return(NOTIFY_DONE);
}

/* Handle iconify signal from another application */

static void icon_func(action)
int action;
{
	int status;

	status = (int) xv_get(VAAFrame,FRAME_CLOSED);
	if (status == action) return;
	if (!action) {
		xv_set(VAAFrame,FRAME_CLOSED,FALSE,NULL);
	}
	else {
		xv_set(VAAFrame,FRAME_CLOSED,TRUE,NULL);
	}
}

Notify_value event_interposer(frame,event,arg,type)
Frame frame;
Event *event;
Notify_arg arg;
Notify_event_type type;
{
	int before,after;
	Notify_value value;

	if (event_id(event) == WIN_RESIZE) {
		value = notify_next_event_func(frame,event,arg,type);
	}
	else if (event_id(event) == ACTION_NULL_EVENT) {
		before = (int) xv_get(frame,FRAME_CLOSED);
		value = notify_next_event_func(frame,event,arg,type);
		after = (int) xv_get(frame,FRAME_CLOSED);
		if (before != after) {
			;
		}
	}
	else value = notify_next_event_func(frame,event,arg,type);
	return(value);
}

SetupVideo()
{
	int i;
	FILE *fp;

	if ((fp = fopen(filename,"r")) == NULL) {
		ShowAlert("Could not open file:",filename,NULL);
		return;
	}
	i = fscanf(fp,"%s%d%s%d%s",Title,&StartFrame,Title,&EndFrame,Title);
	if (i != 5) {
		ShowAlert("Could not parse video clip file",filename,NULL);
		return;
	}

	fgets(Title,80,fp);
	if (strlen(Title) == 0) {
		ShowAlert("Could not parse video clip file",filename,NULL);
		return;
	}
	xv_set(VAAFrame,XV_LABEL,Title,NULL);

	clearerr(fp);
	textsw_erase(VideoText,0,TEXTSW_INFINITY);
	while (feof(fp) == NULL) {
		if (fscanf(fp,"%s",Title) != 1) break;
		fgets(Title,80,fp);
		i = strlen(Title);
		if (i) textsw_insert(VideoText,Title,i);
	}
	fclose(fp);


	if (EndFrame < StartFrame) EndFrame = StartFrame;

	if (StartFrame == EndFrame) ShowDiscFrame(VideoPlayer,StartFrame);
	else ShowDiscSegment(VideoPlayer,StartFrame,EndFrame,FALSE);
}

/* Read the primary selection for name of video file to display */

static Notify_value play_func()
{
	Seln_holder holder;
	Seln_request *result;
	Seln_rank selection;
	char *data,value[1024];

	data		= NULL;
	selection	= SELN_PRIMARY;
	holder		= seln_inquire(selection);
	result		= seln_ask(&holder,SELN_REQ_CONTENTS_ASCII,NULL,NULL);
	data		= result->data + sizeof(SELN_REQ_CONTENTS_ASCII);
	if (*data) {
		     if (strcmp(data,"OPEN") == NULL) icon_func(FALSE);
		else if (strcmp(data,"CLOSE") == NULL) icon_func(TRUE);
		else if (strcmp(data,"STOP") == NULL) {
		}
		else if (strcmp(data,"QUIT") == NULL) {
			xv_destroy_safe(VAAFrame);
		}
		else {
			filename[0] = NULL;
			strcpy(filename,data);
			SetupVideo();
		}
	}
	return(NOTIFY_DONE);
}

/* Event handler for the video pseudo-canvas */

static void event_handler(canvas, event, arg)
Xv_Window canvas;
Event *event;
Notify_arg arg;
{
	Seln_holder holder;
	Seln_request *result;
	Seln_rank selection;
	char *data,value[1024];

	data = NULL;

	switch (event_action(event)) {
	case ACTION_DRAG_LOAD:
		selection = SELN_PRIMARY;
		holder = seln_inquire(selection);
		result = seln_ask(&holder,SELN_REQ_CONTENTS_ASCII,NULL,NULL);
		data = result->data + sizeof(SELN_REQ_CONTENTS_ASCII);
		break;
	}
	if (data) {
		filename[0] = NULL;
		strcpy(filename,data);
		SetupVideo();
	}
}

/* Event function for laserdisc panel to handle all button presses */

static void VideoPanelEvent(item,event)
Panel_item item;
Event *event;
{
	int who;

	who = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	switch (who) {
	case 3:
		if (event_is_down(event)) {
			PlayerSlowReverse(VideoPlayer);
		}
		else {
			PlayerStill(VideoPlayer);
		}
		break;
	case 5:
		if (event_is_down(event)) {
			PlayerSlowForward(VideoPlayer);
		}
		else {
			PlayerStill(VideoPlayer);
		}
		break;
	default:
		break;
	}
	panel_default_handle_event(item,event);
}

/* Notify function for Player panel to handle all button presses */

static void VideoPanelProc(item, event)
Panel_item item;
Event *event;
{
	char *device;
	int speed,model;
	int who;

	who = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	
	switch (who) {
	case 1:
		PlayerFastReverse(VideoPlayer);
		break;
	case 2:
		PlayerReverse(VideoPlayer);
		break;
	case 3:
		break;
	case 4:
		PlayerStill(VideoPlayer);
		break;
	case 5:
		break;
	case 6:
		PlayerPlay(VideoPlayer);
		break;
	case 7:
		PlayerFastForward(VideoPlayer);
		break;
	case 8:
		if (EndFrame < StartFrame) {
			EndFrame = StartFrame;
		}
		if (StartFrame == EndFrame) ShowDiscFrame(VideoPlayer,StartFrame);
		else {
			if (event_shift_is_down(event))
				ShowDiscSegment(VideoPlayer,StartFrame,EndFrame,TRUE);
			else ShowDiscSegment(VideoPlayer,StartFrame,EndFrame,FALSE);
		}
		break;
	}
}

/* Create panel for controlling the video player on the screen */

static int CreatePlayerPanel()
{
	Server_image si;

	VideoPanel = xv_create(VAAFrame,PANEL,
		NULL);

	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			fastreverse_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,1),
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			reverse_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,11),
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			slowreverse_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,21),
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			still_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,31),
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			slowforward_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,41),
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			5,
		NULL);
	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			forward_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,51),
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			6,
		NULL);
	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			fastforward_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,61),
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			7,
		NULL);
	si = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			playsegment_bits,
		NULL);
	(void) xv_create(VideoPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			VideoPanelEvent,
		PANEL_NOTIFY_PROC,			VideoPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(VideoPanel,71)+5,
		XV_Y,						xv_row(VideoPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			8,
		NULL);

	window_fit(VideoPanel);
	return(NULL);
}

/* Main Function for dand.c */
	
main(argc,argv)
int argc;
char *argv[];
{
	FILE *fp;
	int i;
	Icon icon;
	Server_image a,b;

/* Initialize xview toolkit */

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

/* Initialize constants */

	filename[0]			= '\0';

	PlayerModel			= LIVE_VIDEO;
	PlayerSpeed			= NULL;
	PlayerDevice		= "/dev/null";

/* Get options */

	while ((i = getopt(argc, argv, "S:P:N:O:1:2:4:9:d:p?")) != EOF) {
		switch (i) {
		case 'p':
			savepid = TRUE;
			break;
		case 'S':
			PlayerModel = SONY_IP;
			break;
		case 'P':
			PlayerModel = PIONEER;
			break;
		case 'O':
			PlayerModel = PANASONIC;
			break;
		case 'N':
			PlayerModel = NEC;
			break;
		case '1':
			PlayerSpeed = 1;
			break;
		case '2':
			PlayerSpeed = 2;
			break;
		case '4':
			PlayerSpeed = 3;
			break;
		case '9':
			PlayerSpeed = 4;
			break;
		case 'd':
			PlayerDevice = optarg;
			break;
		case '?':
			fprintf(stderr,"usage: %s [ -speed ] [-Sony -Pioneer -NEC -POD] [-d device]\n", argv[0]);
			exit(1);
			break;
		}
	}

	if ((argc - optind) != 0) {
		fprintf(stderr,"usage: %s [ -speed ] [-Sony -Pioneer -NEC -POD] [-d device]\n", argv[0]);
		exit(1);
	}

/* Create our frame icon */

	a = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,				icon_width,
		XV_HEIGHT,				icon_height,
		SERVER_IMAGE_BITS,		icon_bits,
		NULL);

	b = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,				icon_width,
		XV_HEIGHT,				icon_height,
		SERVER_IMAGE_BITS,		mask_bits,
		NULL);

	icon = (Icon) xv_create(VAAFrame,ICON,
		ICON_IMAGE,				a,
		ICON_MASK_IMAGE,		b,
		ICON_TRANSPARENT,		TRUE,
		NULL);

/* create our window */
	
	VAAFrame = xv_create(NULL,FRAME,
		XV_LABEL,					"Untitled",
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_ICON,					icon,
		WIN_CONSUME_EVENT,			WIN_CONFIGURE_REQUEST,
		NULL);

/* Interpose on the frame's resize and destroy events */

	(void) notify_interpose_event_func(VAAFrame,event_interposer,NOTIFY_SAFE);
	(void) notify_interpose_destroy_func(VAAFrame,destroy_interposer);

/* Create the control panel for the user */

	CreatePlayerPanel();

/* Initialize a default video source for this channel */

	VideoPlayer = (Player *) InitializeVideo(PlayerDevice,PlayerModel,PlayerSpeed);

	if (VideoPlayer == NULL) exit(1);

/* Create the video pseudo-canvas */

	VideoCanvas = xv_create(VAAFrame,VIDEO_IN,
		VIDEO_CHANNEL,			VIDEO_IN_ONE,
		VIDEO_SQUEEZE,			TRUE,
		XV_X,					0,
		XV_WIDTH,				640,
		XV_HEIGHT,				482,
		WIN_BELOW,				VideoPanel,
		NULL);

	if (VideoCanvas == NULL) {
		if (Xv_VideoStatus == VIDEO_CHANNEL_IN_USE) {
			ShowAlert("Requested channel already in use",NULL);
		}
		else if (Xv_VideoStatus == VIDEO_NO_MORE_CHANNELS) {
			ShowAlert("Requested channel is not available",NULL);
		}
		else {
			ShowAlert("xv_create of video canvas failed!",NULL);
		}
		return(NULL);
	}

/* Attach an event handler to the video in object */

	xv_set(VideoCanvas,VIDEO_EVENT_PROC,event_handler,NULL);

/* Add the text display area */

	VideoText = (Textsw) xv_create(VAAFrame,TEXTSW,
		XV_X,					0,
		XV_HEIGHT,				100,
		WIN_BELOW,				VideoCanvas,
		NULL);

	window_fit(VAAFrame);

/* Setup our signals for doing special things */

	notify_set_signal_func(VAAFrame,play_func,SIGUSR1,NOTIFY_SYNC);

	if (savepid) {
		if ((fp = fopen("/tmp/dandpid","w")) == NULL) {
			fprintf(stderr,"Unable to create /tmp/dandpid\n");
			exit(1);
		}
		fprintf(fp,"%d",getpid());
		fclose(fp);
	}

	xv_main_loop(VAAFrame);

	if (savepid) unlink("/tmp/dandpid");

	exit(0);
}

/* end of dand.c */

