/***************************************************************************
 *
 *	netvid.c - Network Video Server
 *
 *	(C) Copyright 1991 by Paradise Software, Inc.
 *
 *	Written by John Malleo-Roach
 *
 *	Version %G% - %I%
 *
 ***************************************************************************/

/* INCLUDE FILES */

#include <fcntl.h>
#include <errno.h>
#include <machine/param.h>
#include <stropts.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <sun/tvio.h>
#include <sundev/tv1reg.h>

#include "Video.h"
#include "xview/font.h"

#include <X11/Xlib.h>

#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>

/* CONSTANTS */

/* Size of each packet being sent ( 1/8 frame with 2 pixels per byte plus count) */

#define SIZE		120 * 160 / 16 + 4

#define PANEL_DATA				1000
#define ITEM_DATA				1001

#define AUDIO_DATA				0xffffffff

/* Constants for the audio device */

#define MAX_GAIN				100
#define DEFAULT_RECORD_GAIN  	50
#define DEFAULT_MONITOR_GAIN 	50

#define FOR_VIDEO				0
#define FOR_AUDIO				1

/* Constants for the audio device */

#define DEFAULT_RECORD_GAIN  	50
#define DEFAULT_MONITOR_GAIN 	50

/* STATIC VARIABLES */

static Display  *VAADisplay;		/* Local display, XView Screen and X11 Screen	*/
static Xv_Screen VAAXVScreen;
static int       VAAScreen;

static VideoIn VideoCanvas;			/* Video canvas for displaying input video		*/
static Panel   ConfigPanel;			/* Panel for controling Video parameters		*/
static Panel   AudioPanel;			/* Panel for controlling audio parameters		*/
static Canvas  AudioCanvas;			/* Canvas for displaying audio volume			*/

static Panel_item HueItem;			/* Pointer to panel item hue adjustment			*/
static Panel_item BrightnessItem;	/* Pointer to panel item brightness adjustment	*/
static Panel_item ContrastItem;		/* Pointer to panel item contrast adjustment	*/
static Panel_item SaturationItem;	/* Pointer to panel item saturation adjustment	*/
static Panel_item ChannelItem;		/* Pointer to panel item for video channel		*/
static Panel_item FormatItem;		/* Pointer to panel item video format			*/
static Panel_item TypeItem;			/* Pointer to panel item input type				*/
static Panel_item DefaultSpeed;		/* Pointer to panel item for capture rate		*/
static Panel_item CurrentSpeed;		/* Pointer to panel item current rate			*/
static Panel_item RecordVolume;		/* Pointer to panel item for record volume		*/
static Panel_item MonitorVolume;	/* Pointer to panel item for monitor volume		*/
static Panel_item AudioOut;			/* Pointer to panel item for where audio goes	*/

static          int  VideoFD;
static unsigned int *VideoBuffer;
static          int  VideoSocket;
static			int  RequestSocket;
static unsigned int  FrameNumber=0;
static unsigned int  SequenceNumber=0;

static struct itimerval  Timer;
static struct timeval    tp;
static        char      *hostname;
static		  int		 Speed=1;

static unsigned char Buffer[16384];

static struct sockaddr_in Source;
static struct sockaddr_in Destination;

static int Action[16][16][2] = {
	{
		{ 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 },
		{ 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 }
	},{
		{ 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 },
		{ 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 1, 1 }, { 0, 0 }, { 0, 1 },
		{ 0, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 0, 0 }, { 1, 1 }, { 0, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 },
		{ 0, 0 }, { 1, 1 }, { 0, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 0, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 },
		{ 0, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 0, 0 }, { 1, 1 }, { 0, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 0, 0 }, { 1, 1 }, { 0, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 },
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 },
		{ 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }
	},{
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 },
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }
	},{
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 },
		{ 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 1, 1 }, { 1, 0 }, { 0, 1 }
	}
};

static int ClientTable[256];		/* Store client requests for frame speed		*/

/* AUDIO STATIC VARIABLES */

static u_char AudioBuf[16384];		/* Buffer for storing input data				*/
static int AudioBytes;				/* Number of bytes read during this open		*/

static Audio_hdr RecordHeader;		/* Audio record header for device				*/

static int    AudioFd=0;			/* File descriptor for audio device				*/
static Window AudioXID;				/* X Window associated with AudioCanvas pw		*/
static GC     AudioGCB;				/* Graphics context with Black foreground		*/
static GC     AudioGCR;				/* Graphics context	with Red foreground			*/
static GC     AudioGCW;				/* Graphics context	with White foreground		*/
static int    AudioWidth;			/* Width of audio volume canvas					*/
static int    AudioHeight;			/* Height of audio volume canvas				*/
static int    AudioLevel;			/* Current remembered audio level				*/
static int	  AudioEnable=1;		/* Send audio with the video					*/

/* GLOBALS VARIABLES */

Frame   VAAFrame;					/* Base frame for network video application		*/

/* EXTERNAL DEFINITIONS */

extern int   optind;
extern char *optarg;

/* FUNCTIONS */

/* Pause for a while */

static int nap(msec)
int msec;
{
	int i;
	fd_set list;
	struct timeval timeout;

	timeout.tv_sec = 0;
	timeout.tv_usec = msec * 1000;
	
	FD_ZERO(&list);
	FD_SET(RequestSocket,&list);
	
	i = select(FD_SETSIZE, (fd_set *) &list, (fd_set *) 0, (fd_set *) 0, &timeout);
	if (i < 0) {
		perror("Select failed");
		return(0);
	}
	return(i);
}
    
/* Compute some interesting playback numbers for the user */

void FindRate()
{
	char buf[32];
	float d,e;
	struct timeval tt;
	int i,j;

	gettimeofday(&tt,NULL);
	i = tt.tv_sec * 1000000 + tt.tv_usec;
	j = tp.tv_sec * 1000000 + tp.tv_usec;
	i = i - j;
	e = ((float) i) / 100.0;
	d = 1000000.0 / e;
	sprintf(buf,"   Current Rate :   %1.1f",d);
	xv_set(CurrentSpeed,PANEL_LABEL_STRING,buf,NULL);
	gettimeofday(&tp,NULL);
}

Notify_value quit(client,status)
Notify_client client;
Destroy_status status;
{
	float gain;

	if (status == DESTROY_CLEANUP) {
		gain = 0.0;
		audio_set_monitor_gain(AudioFd,&gain);
		CloseAudio(NULL);
		xv_destroy(VideoCanvas);
		return(notify_next_destroy_func(client,status));
	}
	return(NOTIFY_DONE);
}

/* Catch signals for emergency exit */

Notify_value SignalCatcher(client, sig, when)
Notify_client client;
int sig;
Notify_signal_mode when;
{
	double gain;

	gain = 0.0;
	audio_set_monitor_gain(AudioFd,&gain);
	CloseAudio(NULL);
	xv_destroy_safe(VideoCanvas);
	return(NOTIFY_DONE);
}

/* MMap the video buffer */

static void MapVideo()
{
	if ((VideoFD = open("/dev/tvone0", O_RDONLY)) < 0) {
	    perror("Fatal error opening /dev/tvone0");
	    exit(1);
	}
	if ((VideoBuffer = (unsigned int *) mmap(0,FB_SIZE,PROT_READ,MAP_SHARED,
		VideoFD,FB_BASE|0x80000000)) == (unsigned int *) -1) {
		    perror("Fatal error mapping frame buffer");
		    exit(1);
	}
}

/* Take one frame off the video buffer */

CaptureFrame()
{
	unsigned int *a;
	register int  b,d,e,f;
	register unsigned char *c;
	int i,j,k,l;

	a = VideoBuffer + (30 * 1024);
	bcopy(&FrameNumber,Buffer,sizeof(int));
	c = Buffer + sizeof(int);
	j = (3 * 1024) + (1024 - 320);

	i = 120;
	l = 16;
	while (i) {
		k = 80;
		while (k) {
			b  = *a; a += 2; d  = ((b >>  8) & 0xff); d <<= 1;
			d += b & 0xff;   d += ((b >> 16) & 0xff); e   = d >> 5;
			if (e > l) {
				e -= l; if (e > 8) e = 8; l += e; e += 7;
			}
			else {
				e = l - e; if (e > 7) e = 7; l -= e;
			}
			b  = *a; a += 2; d  = ((b >>  8) & 0xff); d <<= 1;
			d += b & 0xff;   d += ((b >> 16) & 0xff); d   = d >> 5;
			if (d > l) {
				d -= l; if (d > 8) d = 8; l += d; d += 7;
			}
			else {
				d = l - d; if (d > 7) d = 7; l -= d;
			}
			*c++ = (e << 4) | d;
			k--;
		}
		i--; a += j;
		if ((i % 15) == 0) {
			if (sendto(VideoSocket,Buffer,SIZE,NULL,&Destination,sizeof(Destination)) < 0) {
				perror("sendto network");
			}
			FrameNumber++;
			bcopy(&FrameNumber,Buffer,sizeof(int));
			c = Buffer + sizeof(int);
			l = 16;
		}
	}
}

/* Get a new frame of video from the source */

static Notify_value GrabFrame()
{
	CaptureFrame();

/*	xv_set(VideoCanvas,VIDEO_NEW_FRAME,TRUE,NULL); */

	if ((FrameNumber % 800) == 0) FindRate();

	return(NOTIFY_DONE);
}

/* Open up a local network broadcast connection */

OpenConnection()
{
	int i,j;
	struct netent *net;
	struct hostent *host;
	struct ifconf ifc;
	struct ifreq ifreq, *ifr;
	
/* Connection for sending our video data */

	if ((VideoSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("Socket open");
		exit(1);
	}

	i   = 1;
	if (setsockopt(VideoSocket,SOL_SOCKET, SO_BROADCAST,&i,sizeof(i)) < 0) {
		perror("setsockopt SO_BROADCAST");
		exit(1);
	}

	bzero(&Destination,sizeof(Destination));
	Destination.sin_family = AF_INET;
	Destination.sin_port   = htons(IPPORT_USERRESERVED);

	if (strcmp(hostname,"broadcast")) {
		host = gethostbyname(hostname);
		bcopy(host->h_addr,&(Destination.sin_addr),host->h_length);
	}
	else {
		if (bind(VideoSocket,&Destination,sizeof(Destination)) < 0) {
			perror("bind socket");
			exit(1);
		}
		net  = getnetbyname("localnet");
		Destination.sin_addr   = inet_makeaddr(net->n_net,INADDR_BROADCAST);
		Destination.sin_addr.S_un.S_un_b.s_b4 = 0;
		Destination.sin_addr.S_un.S_un_b.s_b3 = 0;
		Destination.sin_addr.S_un.S_un_b.s_b2 = 0;
	}

/* Up the output buffer size */

	i   = 32767;
	if (setsockopt(VideoSocket,SOL_SOCKET,SO_SNDBUF,&i,sizeof(int)) < 0) {
		perror("setsockopt SO_SND");
		exit(1);
	}

/* Connection for listening for bandwidth requests */

	if ((RequestSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("Socket open");
		exit(1);
	}
	bzero(&Source,sizeof(Source));
	Source.sin_family      = AF_INET;
	Source.sin_addr.s_addr = htonl(INADDR_ANY);
	Source.sin_port        = htons(IPPORT_USERRESERVED+1);
	if (bind(RequestSocket,&Source,sizeof(Source)) < 0) {
		perror("bind socket");
		exit(1);
	}
}

HandleRequest()
{
	int i,j;

	i = sizeof(Source);
	recvfrom(RequestSocket,&j,sizeof(int),NULL,&Source,&i);
#ifdef DEBUG
	printf("Recieved request (%d) from %s\n",j,inet_ntoa(Source.sin_addr));
#endif
	i = Destination.sin_addr.S_un.S_un_b.s_b4;
	ClientTable[i] = j;
}

/* Open up the audio device for playing and recording */

int OpenAudio()
{
	struct stat info;
	int flag,value;
	double gain;

	if ((AudioFd = open("/dev/audio",O_RDONLY|O_NDELAY)) < 0) {
		if ((errno == EBUSY) || (errno == EINTR)) {
			ShowNote("Audio device is already open by another process",NULL);
		}
		else perror("Could not open audio device");
		return(FALSE);
	}

/* Set up the audio device for asynchronous, non-blocking reads */

	flag = fcntl(AudioFd, F_GETFL, 0) | FNDELAY;
    if (fcntl(AudioFd, F_SETFL, flag) < 0) {
		perror("F_SETFL fcntl failed");
		close(AudioFd);
		return(FALSE);
	}
	if (ioctl(AudioFd,I_SETSIG,S_INPUT|S_OUTPUT|S_MSG) < 0) {
		perror("I_SETSIG ioctl failed");
		close(AudioFd);
		return(FALSE);
	}

/* Get encoding information from device */

	if (audio_get_record_config(AudioFd,&RecordHeader) != AUDIO_SUCCESS) {
		ShowAlert("Cound not get record encoding configuration",NULL);
		return(FALSE);
	}

/* Set default play, record, monitor values */

	gain = (double) 0.0;
	audio_set_play_gain(AudioFd,&gain);
	value = (int) xv_get(RecordVolume,PANEL_VALUE);
	gain = ((double) value) / ((double) MAX_GAIN);
	audio_set_record_gain(AudioFd,&gain);
	value = (int) xv_get(MonitorVolume,PANEL_VALUE);
	gain = ((double) value) / ((double) MAX_GAIN);
	audio_set_monitor_gain(AudioFd,&gain);
	value = (int) xv_get(AudioOut,PANEL_VALUE);
	(void) audio_set_play_port(AudioFd,&value);
	AudioBytes = 0;
	return(TRUE);
}

/* Grab whatever audio we can */

GrabAudio()
{
	u_char *c;
	u_int i;
	int size;

	size = 0;
	if (ioctl(AudioFd,FIONREAD,&size) < 0) {
		perror("FIONREAD ioctl failed");
		return;
	}

	if (size < 16) return;
	if (size > 1200) size = 1200;

	i = AUDIO_DATA;
	bcopy(&i,AudioBuf,sizeof(int));
	c = AudioBuf + sizeof(int);

	if ((i = read(AudioFd,c,size)) < 0) {
		if (errno != EWOULDBLOCK) {
			perror("Audio read failed");
			return;
		}
	}
	if (AudioEnable) {
		if (sendto(VideoSocket,c,i,NULL,&Destination,sizeof(Destination)) < 0) {
			perror("sendto network");
		}
	}
	return(i);
}

int CloseAudio()
{
	int err;

	audio_pause_record(AudioFd);
	audio_flush_record(AudioFd);
	close(AudioFd); AudioFd = 0;
}

/* Draw our floating bar volume display */

void UpdateAudioDisplay()
{
	int width,height;
	int i,j,value;
	int x,y;

	value = 0;
	for (i=0; i<1024; i=i+64) {
		j = (int) AudioBuf[0];
		if (j & 128) j = 255 - j;
		else j = 128 - j;
		value += j;
	}
	value = value / 116;
	AudioLevel += (value - AudioLevel) / 2;
		
	XFillRectangle(VAADisplay,AudioXID,AudioGCB,20,0,AudioWidth,AudioHeight);

	width  = AudioWidth - 40;
	y      = AudioHeight / 16;
	height = AudioHeight / 18;

	if (AudioLevel < 9) {
		for (i=0; i<AudioLevel; i++) {
			XFillRectangle(VAADisplay,AudioXID,AudioGCW,30,AudioHeight-y*i,width,height);
		}
	}
	else {
		for (i=0; i<11; i++) {
			XFillRectangle(VAADisplay,AudioXID,AudioGCW,30,AudioHeight-y*i,width,height);
		}
		for (; i<AudioLevel; i++) {
			XFillRectangle(VAADisplay,AudioXID,AudioGCR,30,AudioHeight-y*i,width,height);
		}
	}
}

/* Panel notify function for the audio panel */

void AudioPanelProc(item, value, event)
Panel_item item;
int value;
Event *event;
{
	int who;
	unsigned port;
	double gain;

	who		= (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	
	switch (who) {
	case 1:
		port = (value == 0 ? AUDIO_SPEAKER : AUDIO_HEADPHONE);
		(void) audio_set_play_port(AudioFd,&port);
		break;
	case 2:
		gain = ((double) value) / ((double) MAX_GAIN);
		audio_set_record_gain(AudioFd,&gain);
		break;
	case 3:
		gain = ((double) value) / ((double) MAX_GAIN);
		audio_set_monitor_gain(AudioFd,&gain);
		break;
	case 4:
		AudioEnable = value;
		break;
	}
}

/* Repaint proc for audio canvas */

void AudioRepaintProc(canvas,pw,rects)
Canvas canvas;
Xv_Window pw;
Rectlist *rects;
{
	int step,start;

	XFillRectangle(VAADisplay,AudioXID,AudioGCB,0,0,AudioWidth,AudioHeight);

	step  = AudioHeight / 11;
	start = AudioHeight - 5;

	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"DB",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"40",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"30",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start,"20",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start," 5",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start," 2",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCW,5,start," 0",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start," 2",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start," 5",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start,"10",2);
	start -= step;
	XDrawString(VAADisplay,AudioXID,AudioGCR,5,start," +",2);
}

void ConfigPanelProc(item, value)
Panel_item item;
int value;
{
	Xv_opaque parent;
	int who,what,i;

	who	   = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	parent = (Xv_opaque) xv_get(item,XV_OWNER);
	parent = (Xv_opaque) xv_get(parent,XV_OWNER);

	xv_set(parent,FRAME_BUSY,TRUE,NULL);

	switch (who) {
	case 1:
		xv_set(VideoCanvas,VIDEO_HUE,value,NULL);
		break;
	case 2:
		xv_set(VideoCanvas,VIDEO_BRIGHTNESS,value,NULL);
		break;
	case 3:
		xv_set(VideoCanvas,VIDEO_CONTRAST,value,NULL);
		break;
	case 4:
		xv_set(VideoCanvas,VIDEO_SATURATION,value,NULL);
		break;
	case 5:
		(void) xv_get(VideoCanvas,VIDEO_FACTORY_RESET);
		what = (int) xv_get(VideoCanvas,VIDEO_HUE);
		xv_set(HueItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(VideoCanvas,VIDEO_BRIGHTNESS);
		xv_set(BrightnessItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(VideoCanvas,VIDEO_CONTRAST);
		xv_set(ContrastItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(VideoCanvas,VIDEO_SATURATION);
		xv_set(SaturationItem,PANEL_VALUE,what,NULL);
		break;
	case 6:
		who = (int) xv_get(VideoCanvas,VIDEO_CHANNEL);
		switch (who) {
		case VIDEO_IN_ONE:
			xv_set(VideoCanvas,VIDEO_USER0,TRUE,NULL);
			break;
		case VIDEO_IN_TWO:
			xv_set(VideoCanvas,VIDEO_USER1,TRUE,NULL);
			break;
		}
		break;
	case 7:
		who = (int) xv_get(VideoCanvas,VIDEO_CHANNEL);
		switch (who) {
		case VIDEO_IN_ONE:
			(void) xv_get(VideoCanvas,VIDEO_USER0);
			break;
		case VIDEO_IN_TWO:
			(void) xv_get(VideoCanvas,VIDEO_USER1);
			break;
		}
		what = (int) xv_get(VideoCanvas,VIDEO_HUE);
		xv_set(HueItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(VideoCanvas,VIDEO_BRIGHTNESS);
		xv_set(BrightnessItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(VideoCanvas,VIDEO_CONTRAST);
		xv_set(ContrastItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(VideoCanvas,VIDEO_SATURATION);
		xv_set(SaturationItem,PANEL_VALUE,what,NULL);
		break;
	case 8:
		switch (value) {
		case 0:
			what = xv_set(VideoCanvas,VIDEO_CHANNEL,VIDEO_IN_ONE,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_NO_MORE_CHANNELS) {
					ShowAlert("This channel is not supported",NULL);
				}
				else if (Xv_VideoStatus == VIDEO_CHANNEL_IN_USE) {
					ShowAlert("This channel is already in use",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value);
			break;
		case 1:
			what = xv_set(VideoCanvas,VIDEO_CHANNEL,VIDEO_IN_TWO,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_NO_MORE_CHANNELS) {
					ShowAlert("This channel is not supported",NULL);
				}
				else if (Xv_VideoStatus == VIDEO_CHANNEL_IN_USE) {
					ShowAlert("This channel is already in use",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value);
			break;
		}
		break;
	case 9:
		switch (value) {
		case 0:
			what = xv_set(VideoCanvas,VIDEO_FORMAT,VIDEO_COMPOSITE,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value);
			break;
		case 1:
			what = xv_set(VideoCanvas,VIDEO_FORMAT,VIDEO_SUPER_VHS,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 2:
			what = xv_set(VideoCanvas,VIDEO_FORMAT,VIDEO_YUV,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 3:
			what = xv_set(VideoCanvas,VIDEO_FORMAT,VIDEO_RGB,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		}
		break;
	case 10:
		switch (value) {
		case 0:
			what = xv_set(VideoCanvas,VIDEO_INPUT_TYPE,VIDEO_YUV7,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 1:
			what = xv_set(VideoCanvas,VIDEO_INPUT_TYPE,VIDEO_RGB24,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 2:
			what = xv_set(VideoCanvas,VIDEO_INPUT_TYPE,VIDEO_RGB8,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 3:
			what = xv_set(VideoCanvas,VIDEO_INPUT_TYPE,VIDEO_GRAY7,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 4:
			what = xv_set(VideoCanvas,VIDEO_INPUT_TYPE,VIDEO_GRAY8,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		}
		break;
	case 11:
		Speed = (int) xv_get(item,PANEL_VALUE);
		break;
	}
	xv_set(parent,FRAME_BUSY,FALSE,NULL);
}

/* Create panel for controlling the video colors */

static void CreateConfigPanel()
{
	ConfigPanel = xv_create(VAAFrame,PANEL,NULL);
	if (ConfigPanel == NULL) {
		ShowAlert("xv_create of color control panel failed!",NULL);
		exit(1);
	}
	HueItem = (Panel_item) xv_create(ConfigPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,1),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			"            Hue : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	BrightnessItem = (Panel_item) xv_create(ConfigPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,2),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			"Brightness : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	ContrastItem = (Panel_item) xv_create(ConfigPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,3),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			"    Contrast : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	SaturationItem = (Panel_item) xv_create(ConfigPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,4),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			" Saturation : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	(void) xv_create(ConfigPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_STRING,			"Factory",
		XV_X,						xv_col(ConfigPanel,8),
		XV_Y,						xv_row(ConfigPanel,5)+10,
		XV_KEY_DATA,				PANEL_DATA,			5,
		NULL);
	(void) xv_create(ConfigPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_STRING,			" Save  ",
		XV_X,						xv_col(ConfigPanel,19),
		XV_Y,						xv_row(ConfigPanel,5)+10,
		XV_KEY_DATA,				PANEL_DATA,			6,
		NULL);
	(void) xv_create(ConfigPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_STRING,			"Recall ",
		XV_X,						xv_col(ConfigPanel,30),
		XV_Y,						xv_row(ConfigPanel,5)+10,
		XV_KEY_DATA,				PANEL_DATA,			7,
		NULL);
	ChannelItem = (Panel_item) xv_create(ConfigPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(ConfigPanel,48),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,1)-5,
		PANEL_LABEL_STRING,			"Video Channel : ",
		PANEL_CHOICE_STRINGS,		"ONE","TWO",NULL,
		XV_KEY_DATA,				PANEL_DATA,			8,
		XV_KEY_DATA,				ITEM_DATA,			0,
		NULL);
	FormatItem = (Panel_item) xv_create(ConfigPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,48),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,2)-5,
		PANEL_LABEL_STRING,			"  Video Format : ",
		PANEL_CHOICE_STRINGS,		"Composite","SuperVHS","YUV","RGB",NULL,
		XV_KEY_DATA,				PANEL_DATA,			9,
		XV_KEY_DATA,				ITEM_DATA,			0,
		NULL);
	TypeItem = (Panel_item) xv_create(ConfigPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,48),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,3)-5,
		PANEL_LABEL_STRING,			"     Input Type : ",
		PANEL_CHOICE_STRINGS,		"7 bit YUV","24 bit RGB","8 bit RGB",
			"7 bit GRAY","8 bit GRAY",NULL,
		XV_KEY_DATA,				PANEL_DATA,			10,
		XV_KEY_DATA,				ITEM_DATA,			0,
		NULL);
	DefaultSpeed = xv_create(ConfigPanel, PANEL_NUMERIC_TEXT,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,48),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,4),
		PANEL_LABEL_STRING,			"   Default Rate : ",
		PANEL_VALUE_DISPLAY_LENGTH,	2,
		PANEL_VALUE,				Speed,
		PANEL_MAX_VALUE,			15,
		XV_KEY_DATA,				PANEL_DATA,			11,
		NULL);
	CurrentSpeed = xv_create(ConfigPanel, PANEL_MESSAGE,
		PANEL_LABEL_X,				xv_col(ConfigPanel,48),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,5),
		PANEL_LABEL_STRING,			"   Current Rate :   1.0",
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	window_fit(ConfigPanel);
}

/* Create panel for controlling the audio source on the screen */

static int CreateAudioPanel()
{
	Xv_Font font;
	XGCValues gcv;
	int height;

	AudioPanel = xv_create(VAAFrame,PANEL,
		XV_X,				0,
		WIN_BELOW,			ConfigPanel,
		NULL);
	if (AudioPanel == NULL) {
		ShowAlert("xv_create of audio panel failed!",NULL);
		exit(1);
	}
	(void) xv_create(AudioPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_LAYOUT,				PANEL_VERTICAL,
		PANEL_LABEL_X,				xv_col(AudioPanel,0),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2),
		PANEL_LABEL_STRING,			"Audio",
		PANEL_CHOICE_STRINGS,		"Off","On",NULL,
		PANEL_VALUE,				1,
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	AudioOut = xv_create(AudioPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_LAYOUT,				PANEL_VERTICAL,
		PANEL_LABEL_X,				xv_col(AudioPanel,0),
		PANEL_LABEL_Y,				xv_row(AudioPanel,4),
		PANEL_LABEL_STRING,			"Audio Out",
		PANEL_CHOICE_STRINGS,		"Speaker","External",NULL,
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	RecordVolume = xv_create(AudioPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_LABEL_X,				xv_col(AudioPanel,15),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			100,
		PANEL_VALUE,				DEFAULT_RECORD_GAIN,
		PANEL_DIRECTION,			PANEL_VERTICAL,
		PANEL_SLIDER_END_BOXES,		TRUE,
		PANEL_TICKS,				11,
		PANEL_SHOW_VALUE,			FALSE,
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"R",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+16,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"E",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+28,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"C",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+40,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"O",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+52,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"R",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+64,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"D",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+76,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	MonitorVolume = xv_create(AudioPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_LABEL_X,				xv_col(AudioPanel,25),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			100,
		PANEL_VALUE,				DEFAULT_MONITOR_GAIN,
		PANEL_DIRECTION,			PANEL_VERTICAL,
		PANEL_SLIDER_END_BOXES,		TRUE,
		PANEL_TICKS,				11,
		PANEL_SHOW_VALUE,			FALSE,
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"M",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+12,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"O",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+24,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"N",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+36,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"I",
		PANEL_LABEL_X,				xv_col(AudioPanel,22)+2,
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+48,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"T",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+60,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"O",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+72,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"R",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2)+84,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);

	window_fit(AudioPanel);

	height = xv_get(AudioPanel,XV_HEIGHT);
	AudioCanvas = xv_create(VAAFrame,CANVAS,
		CANVAS_WIDTH,				70,
		CANVAS_HEIGHT,				height,
		CANVAS_RETAINED,			FALSE,
		CANVAS_REPAINT_PROC,		AudioRepaintProc,
		XV_Y,						0,
		XV_WIDTH,					70,
		XV_HEIGHT,					height,
		WIN_RIGHT_OF,				AudioPanel,
		WIN_BELOW,					ConfigPanel,
		NULL);

	window_fit(AudioCanvas);

/* FIXUP: height moves for video window
	AudioWidth     = (int) xv_get(AudioCanvas,XV_WIDTH);
	AudioHeight    = (int) xv_get(AudioCanvas,XV_HEIGHT);
*/
	AudioWidth	   = 70;
	AudioHeight    = 240;

	AudioXID       = (Window) xv_get(canvas_paint_window(AudioCanvas),XV_XID);
	AudioLevel	   = 0;
	font = (Xv_Font) xv_find(VAAFrame,FONT,
		FONT_FAMILY,		FONT_FAMILY_DEFAULT,
		FONT_STYLE,			FONT_STYLE_BOLD,
		NULL);

	gcv.function   = GXcopy;
	gcv.font       = (Font) xv_get(font,XV_XID);
	gcv.foreground = WhitePixel(VAADisplay,VAAScreen);
	gcv.background = BlackPixel(VAADisplay,VAAScreen);
	AudioGCW       = XCreateGC(VAADisplay,AudioXID,
						GCFunction|GCForeground|GCBackground,&gcv);
	gcv.foreground = BlackPixel(VAADisplay,VAAScreen);
	gcv.background = WhitePixel(VAADisplay,VAAScreen);
	AudioGCB       = XCreateGC(VAADisplay,AudioXID,
						GCFunction|GCForeground|GCBackground|GCFont,&gcv);

/* An index into the colormap which is our purest red */

	gcv.foreground = 12;
	AudioGCR       = XCreateGC(VAADisplay,AudioXID,
						GCFunction|GCForeground|GCBackground,&gcv);

	return(NULL);
}

/* Create canvas for displaying video frames from the video board */

static void CreateVideoInput(channel)
int channel;
{
	int h,value;

/* Create a video canvas for display off the board */

	VideoCanvas = (VideoIn) xv_create(VAAFrame,VIDEO_IN,
		XV_WIDTH,				320,
		XV_HEIGHT,				240,
		WIN_BELOW,				ConfigPanel,
/*		VIDEO_LIVE,				FALSE, */
		VIDEO_SQUEEZE,			TRUE,
		NULL);

	if (VideoCanvas == NULL) {
		if (Xv_VideoStatus == VIDEO_CHANNEL_IN_USE) {
			ShowAlert("Requested channel already in use",NULL);
		}
		else if (Xv_VideoStatus == VIDEO_NO_MORE_CHANNELS) {
			ShowAlert("Requested channel is not available",NULL);
		}
		else {
			ShowAlert("xv_create of video canvas failed!",NULL);
		}
	}

/* Set the configuration panel values to current */

	value = (int) xv_get(VideoCanvas,VIDEO_FORMAT);
	switch (value) {
	case VIDEO_COMPOSITE:
		xv_set(FormatItem,PANEL_VALUE,0,NULL);
		xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	case VIDEO_SUPER_VHS:
		xv_set(FormatItem,PANEL_VALUE,1,NULL);
		xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,1,NULL);
		break;
	case VIDEO_YUV:
		xv_set(FormatItem,PANEL_VALUE,2,NULL);
		xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,2,NULL);
		break;
	case VIDEO_RGB:
		xv_set(FormatItem,PANEL_VALUE,3,NULL);
		xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,3,NULL);
		break;
	default:
		xv_set(FormatItem,PANEL_VALUE,0,NULL);
		xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	}

	value = (int) xv_get(VideoCanvas,VIDEO_INPUT_TYPE);
	switch (value) {
	case VIDEO_YUV7:
		xv_set(TypeItem,PANEL_VALUE,0,NULL);
		xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	case VIDEO_RGB24:
		xv_set(TypeItem,PANEL_VALUE,1,NULL);
		xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,1,NULL);
		break;
	case VIDEO_RGB8:
		xv_set(TypeItem,PANEL_VALUE,2,NULL);
		xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,2,NULL);
		break;
	case VIDEO_GRAY7:
		xv_set(TypeItem,PANEL_VALUE,3,NULL);
		xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,3,NULL);
		break;
	case VIDEO_GRAY8:
		xv_set(TypeItem,PANEL_VALUE,4,NULL);
		xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,4,NULL);
		break;
	default:
		xv_set(TypeItem,PANEL_VALUE,0,NULL);
		xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	}

	value = (int) xv_get(VideoCanvas,VIDEO_HUE);
	xv_set(HueItem,PANEL_VALUE,value,NULL);
	value = (int) xv_get(VideoCanvas,VIDEO_BRIGHTNESS);
	xv_set(BrightnessItem,PANEL_VALUE,value,NULL);
	value = (int) xv_get(VideoCanvas,VIDEO_CONTRAST);
	xv_set(ContrastItem,PANEL_VALUE,value,NULL);
	value = (int) xv_get(VideoCanvas,VIDEO_SATURATION);
	xv_set(SaturationItem,PANEL_VALUE,value,NULL);

/* What kind of board is this */

	h = (int) xv_get(VideoCanvas,VIDEO_HARDWARE);
	switch (h) {
	case VIDEO_1280:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"1280",NULL);
		break;
	case VIDEO_VIPER:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"Viper",NULL);
		break;
	case VIDEO_VIDEOVIEW:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"VideoView",NULL);
		break;
	case VIDEO_XVIDEO_24:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"XVideo-24",NULL);
		break;
	case VIDEO_XVIDEO_8:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"XVideo-8",NULL);
		break;
	default:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"Unknown",NULL);
		break;
	}
}

/* MAIN PROGRAM */

main(argc, argv)
int argc;
char *argv[];
{
	int i,j;

/* Initialize the xview toolkit */

    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

	hostname = "broadcast";

/* Grab off any user supplied arguments */

	while ((i = getopt(argc, argv, "h:s:?")) != EOF) {
		switch (i) {
		case 'h':
			hostname = optarg;
			break;
		case 's':
			Speed = atoi(optarg);
			if (Speed > 15) Speed = 15;
			break;
		case '?':
			fprintf(stderr,"usage: netvid [ -s speed ] [ -h hostname ]\n");
			exit(1);
			break;
		}
	}

/* Create local video display */

	VAAFrame = (Frame) xv_create(NULL,FRAME,
		FRAME_SHOW_FOOTER,	TRUE,
		XV_LABEL,			"Network Video Server",
		XV_WIDTH,			320,
		XV_HEIGHT,			240,
		NULL);

	notify_interpose_destroy_func(VAAFrame,quit);

/* Need to know when to close video channels on bad signals */

	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGINT, NOTIFY_ASYNC);
	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGBUS, NOTIFY_ASYNC);
	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGHUP, NOTIFY_ASYNC);
	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGSEGV, NOTIFY_ASYNC);

/* Get X Window important values */

	VAADisplay  = (Display *) xv_get(VAAFrame,XV_DISPLAY);
	VAAXVScreen = (Xv_Screen) xv_get(VAAFrame,XV_SCREEN);
	VAAScreen   = (int) xv_get(VAAXVScreen,SCREEN_NUMBER);

	CreateConfigPanel();
	window_fit(VAAFrame);
	CreateAudioPanel();
	window_fit(VAAFrame);

/* Create video input canvas */

	CreateVideoInput(VIDEO_IN_ONE);

	window_fit(VAAFrame);

/* Map in the video buffer */

	MapVideo();

/* Open up a network connection */

	OpenConnection();

/* Initialize the client frame speed table */

	for (i=0; i<256; i++) ClientTable[i] = Speed;

/* Start listening to the audio coming in */

	OpenAudio();

/* Get the current time of day */

	gettimeofday(&tp,NULL);

/* Our own event dispatching for XView events */

	xv_set(VAAFrame,WIN_SHOW,TRUE,NULL);

	while (TRUE) {
		if (SequenceNumber == 0) {
			j = (int) xv_get(DefaultSpeed,PANEL_VALUE);
			for (i=0; i<256; i++) {
				if (ClientTable[i] > j) j = ClientTable[i];
			}
			Speed = j;
		}
		if (AudioFd == 0) break;
		if (Action[Speed][SequenceNumber][FOR_VIDEO]) GrabFrame();
		else {
			if (nap(62)) HandleRequest();
		}
		if (Action[Speed][SequenceNumber][FOR_AUDIO]) {
			UpdateAudioDisplay(GrabAudio());
		}
		SequenceNumber = (SequenceNumber + 1) & 0xf;
		notify_dispatch();
		if (QLength(VAADisplay))
			xv_input_pending(VAADisplay,XConnectionNumber(VAADisplay));
		XFlush(VAADisplay);
	}
}
