/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:	video.c,v $
 * Revision 1.11  93/06/04  16:38:59  schen
 * add an extra parameter to jpeg_play_movie.
 * 
 * Revision 1.10  93/03/15  16:59:51  schen
 * new release from Paradise Software.
 * 
 * Revision 1.2  91/08/19  13:09:57  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: video.c,v 1.11 93/06/04 16:38:59 schen Exp $ Parallax Graphics Inc";
#endif

/*
 *
 *	video.c - Video support routines for vaa.c
 *
 *	Written by John Malleo-Roach
 *
 */

/* #define SOLID */

#define ALTERNATE		/* Use the parallax alternate way of getting video params   */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<math.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/param.h>

#include	"vaa.h"
#include	"audio.h"
#include	"jpeg.h"

#include	<xview/fullscreen.h>

#include	<X11/cursorfont.h>

/* GLOBAL VARIABLES		*/

static Fullscreen FS;				/* Structure for accessing the full screen		*/
static int SelectMode=1;			/* Use regions or windows to select output		*/

/* EXTERNAL DEFINITIONS */

/* FUNCTIONS */
          
/* Initialize the Video board */

int InitializeHardware()
{
	char buffer[32];

	xv_set(VAAFrame,FRAME_RIGHT_FOOTER,NULL,NULL);

	return(NULL);
}

static int LoadSunRasterFile(path)
char *path;
{
	Menu menu;
	Pixrect *p;
	FILE *fp;

	fp = fopen(path,"r");
	p  = pr_load(fp,NULL);
	if (p == NULL) {
		ShowAlert("Unrecognized file format!","",path,NULL);
		fclose(fp); return(TRUE);
	}
	fclose(fp);
	if (xv_get(CurrentContext->video,VIDEO_LIVE)) {
		xv_set(VideoItem,PANEL_VALUE,1,NULL);
		xv_set(CurrentContext->video,VIDEO_LIVE,FALSE,NULL);
		xv_set(InterlaceButton,PANEL_INACTIVE,FALSE,NULL);
		menu = (Menu) xv_get(FileButton,PANEL_ITEM_MENU);
		xv_set(xv_get(menu,MENU_NTH_ITEM,2),MENU_INACTIVE,FALSE,NULL);
	}
	xv_set(CurrentContext->video,VIDEO_PIXRECT,p,NULL);
	pr_destroy(p); return(FALSE);
}

static int LoadJPEGFile(path)
char *path;
{
	Menu menu;
	JPEGImage *p;

	if ((p = jpeg_load_still(VAADisplay,path)) == NULL) {
		ShowAlert("Single frame JPEG movie load failed!",NULL);
		return(TRUE);
	}
	if (xv_get(CurrentContext->video,VIDEO_LIVE)) {
		xv_set(VideoItem,PANEL_VALUE,1,NULL);
		xv_set(CurrentContext->video,VIDEO_LIVE,FALSE,NULL);
		xv_set(InterlaceButton,PANEL_INACTIVE,FALSE,NULL);
		menu = (Menu) xv_get(FileButton,PANEL_ITEM_MENU);
		xv_set(xv_get(menu,MENU_NTH_ITEM,2),MENU_INACTIVE,FALSE,NULL);
	}
	xv_set(CurrentContext->video,VIDEO_JPEG_IMAGE,p,NULL);
	jpeg_free_image(VAADisplay,p);
	return(FALSE);
}

static int LoadJFIFFile(path)
char *path;
{
	Menu menu;
	JPEGImage *p;

	if ((p = jpeg_load_jfif(VAADisplay,path)) == NULL) {
		ShowAlert("Load of image from JFIF file failed!",NULL);
		return(TRUE);
	}
	if (xv_get(CurrentContext->video,VIDEO_LIVE)) {
		xv_set(VideoItem,PANEL_VALUE,1,NULL);
		xv_set(CurrentContext->video,VIDEO_LIVE,FALSE,NULL);
		xv_set(InterlaceButton,PANEL_INACTIVE,FALSE,NULL);
		menu = (Menu) xv_get(FileButton,PANEL_ITEM_MENU);
		xv_set(xv_get(menu,MENU_NTH_ITEM,2),MENU_INACTIVE,FALSE,NULL);
	}
	xv_set(CurrentContext->video,VIDEO_JPEG_IMAGE,p,NULL);
	jpeg_free_image(VAADisplay,p);
	return(FALSE);
}

void LoadStillFrame(dir,file)
char *dir;
char *file;
{
	FILE *fp;
	char *name;
	int magic;

	if (dir == NULL) dir = "./";
	else if (strlen(dir) == 0) dir = "./";
	if (file == NULL) file = "image";
	else if (strlen(file) == 0) file = "image";
	name = (char *) malloc(strlen(dir) + strlen(file) + 2);
	sprintf(name,"%s/%s",dir,file);
	if ((fp = fopen(name,"r")) == NULL) {
		ShowAlert("Unable to open requested input file","",name,NULL);
		free(name); return;
	}
	magic = getc(fp); fclose(fp);
	     if (magic == (int) 0xff) LoadJFIFFile(name);
	else if (magic == (int)  'j') LoadJPEGFile(name);
	else LoadSunRasterFile(name);
	free(name);
}

void LoadJPEGMovie(dir,file)
char *dir;
char *file;
{
	char *name;

	if (dir == NULL) dir = "./";
	else if (strlen(dir) == 0) dir = "./";
	if (file == NULL) file = "image";
	else if (strlen(file) == 0) file = "image";
	name = (char *) malloc(strlen(dir) + strlen(file) + 2);
	sprintf(name,"%s/%s",dir,file);

	jpeg_play_movie(VAADisplay,CurrentContext->video,name,0);

	free(name);
}

void SaveJPEGFile(dir,file)
char *dir;
char *file;
{
	JPEGImage *p;
	char *name;

	p = (JPEGImage *) xv_get(CurrentContext->video,VIDEO_JPEG_IMAGE);
	if (p == NULL) {
		ShowAlert("Unable to grab JPEG frame from video window",NULL);
		return;
	}
	if (dir == NULL) dir = "./";
	else if (strlen(dir) == 0) dir = "./";
	if (file == NULL) file = "image";
	else if (strlen(file) == 0) file = "image";
	name = (char *) malloc(strlen(dir) + strlen(file) + 2);
	sprintf(name,"%s/%s",dir,file);
	if (jpeg_save_still(VAADisplay,p,name) < 0) {
		ShowAlert("Unable to save JPEG frame into file",NULL);
	}
	free(name);
	jpeg_free_image(VAADisplay,p);
}

void SaveJFIFFile(dir,file)
char *dir;
char *file;
{
	u_char *table;
	JPEGImage *p;
	char *name;

/* Get compressed image */
	p     = (JPEGImage *) xv_get(CurrentContext->video,VIDEO_JPEG_IMAGE);
	if (p == NULL) {
		ShowAlert("Unable to grab JPEG frame from video window",NULL);
		return;
	}
/* Get quantization tables */
	table = (u_char *) xv_get(CurrentContext->video,VIDEO_QTABLE);
	if (table == NULL) {
		ShowAlert("Unable to grab JPEG frame from video window",NULL);
		return;
	}
/* Buile path to file name */
	if (dir == NULL) dir = "./";
	else if (strlen(dir) == 0) dir = "./";
	if (file == NULL) file = "image";
	else if (strlen(file) == 0) file = "image";
	name = (char *) malloc(strlen(dir) + strlen(file) + 2);
	sprintf(name,"%s/%s",dir,file);
	if (jpeg_save_jfif(VAADisplay,p,name,table) < 0) {
		ShowAlert("Unable to save compressed frame into JFIF file",NULL);
	}
	free(name);
	jpeg_free_image(VAADisplay,p);
}

void SaveSunRasterFile(dir,file)
char *dir;
char *file;
{
	Pixrect *p;
	FILE *fp;
	char *name;

	p = (Pixrect *) xv_get(CurrentContext->video,VIDEO_PIXRECT);
	if (p == NULL) {
		ShowAlert("Unable to grab still frame from video window",NULL);
		free(name);
		return;
	}
	if (dir == NULL) dir = "./";
	else if (strlen(dir) == 0) dir = "./";
	if (file == NULL) file = "image";
	else if (strlen(file) == 0) file = "image";
	name = (char *) malloc(strlen(dir) + strlen(file) + 2);
	sprintf(name,"%s/%s",dir,file);
	if ((fp = fopen(name,"w+")) == NULL) {
		ShowAlert("Unable to open output file:",name,"Giving up!",NULL);
		free(name);
		return;
	}
	pr_dump(p,fp,NULL,RT_STANDARD,NULL);
	fclose(fp);
	pr_destroy(p);
	free(name);
}

void ConfigPanelProc(item, value, event)
Panel_item item;
int value;
Event *event;
{
	Menu menu;
	Xv_opaque parent;
	int who,what;
	int w,h;

	if (CurrentContext == NULL) {
		ShowAlert("Please select a video input window for this operation",NULL);
		xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
		return;
	}
	who	   = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	parent = (Xv_opaque) xv_get(item,PANEL_PARENT_PANEL);

	xv_set(parent,FRAME_BUSY,TRUE,NULL);

	switch (who) {
	case 1:
		if (value) {
			xv_set(CurrentContext->video,VIDEO_LIVE,FALSE,NULL);
			xv_set(InterlaceButton,PANEL_INACTIVE,FALSE,NULL);
			menu = (Menu) xv_get(FileButton,PANEL_ITEM_MENU);
			xv_set(xv_get(menu,MENU_NTH_ITEM,2),MENU_INACTIVE,FALSE,NULL);
		}
		else {
			xv_set(CurrentContext->video,VIDEO_LIVE,TRUE,NULL);
			xv_set(InterlaceButton,PANEL_INACTIVE,TRUE,NULL);
			menu = (Menu) xv_get(FileButton,PANEL_ITEM_MENU);
			xv_set(xv_get(menu,MENU_NTH_ITEM,2),MENU_INACTIVE,TRUE,NULL);
		}
		break;
	case 2:
		xv_set(CurrentContext->video,VIDEO_SQUEEZE,value,NULL);
		break;
	case 3:
		w  = (int) xv_get(CurrentContext->video,VIDEO_SIGNAL_WIDTH);
		w -= (int) xv_get(CurrentContext->video,VIDEO_X);
		h  = (int) xv_get(CurrentContext->video,VIDEO_SIGNAL_HEIGHT);
		h -= (int) xv_get(CurrentContext->video,VIDEO_Y);
#ifdef ALTERNATE
		h += 1;
#endif
		xv_set(CurrentContext->frame,
			XV_WIDTH,		w,
			XV_HEIGHT,		h,
			NULL);
		break;
	case 4:
		xv_set(CurrentContext->video,VIDEO_NORMAL_ASPECT,value,NULL);
		break;
	case 5:
		switch (value) {
		case 0:
			what = xv_set(CurrentContext->video,VIDEO_FORMAT,VIDEO_COMPOSITE,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 1:
			what = xv_set(CurrentContext->video,VIDEO_FORMAT,VIDEO_SUPER_VHS,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 2:
			what = xv_set(CurrentContext->video,VIDEO_FORMAT,VIDEO_YUV,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 3:
			what = xv_set(CurrentContext->video,VIDEO_FORMAT,VIDEO_RGB,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		}
		break;
	case 6:
		switch (value) {
		case 0:
			what = xv_set(CurrentContext->video,VIDEO_INPUT_TYPE,VIDEO_RGB24,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 1:
			what = xv_set(CurrentContext->video,VIDEO_INPUT_TYPE,VIDEO_RGB8,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 2:
			what = xv_set(CurrentContext->video,VIDEO_INPUT_TYPE,VIDEO_GRAY8,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 3:
			what = xv_set(CurrentContext->video,VIDEO_INPUT_TYPE,VIDEO_GRAY7,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 4:
			what = xv_set(CurrentContext->video,VIDEO_INPUT_TYPE,VIDEO_YUV7,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_INPUT_TYPE) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		}
		break;
	case 7:
		break;
	case 8:
		w  = (int) xv_get(CurrentContext->video,VIDEO_SIGNAL_WIDTH);
		w -= (int) xv_get(CurrentContext->video,VIDEO_X);
#ifdef ALTERNATE
		w  = w / 2;
#else
		w  = w / 2 + 2;
#endif
		h  = (int) xv_get(CurrentContext->video,VIDEO_SIGNAL_HEIGHT);
		h -= (int) xv_get(CurrentContext->video,VIDEO_Y);
		h  = h / 2 + 1;
		xv_set(CurrentContext->frame,
			XV_WIDTH,		w,
			XV_HEIGHT,		h,
			NULL);
		break;
	}
	xv_set(parent,FRAME_BUSY,FALSE,NULL);
}

void ColorPanelProc(item, value, event)
Panel_item item;
int value;
Event *event;
{
	Menu menu;
	Xv_opaque parent;
	int who,what;
	int w,h;

	if (CurrentContext == NULL) {
		ShowAlert("Please select a video input window for this operation",NULL);
		xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
		return;
	}
	who	   = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	parent = (Xv_opaque) xv_get(item,PANEL_PARENT_PANEL);

	xv_set(parent,FRAME_BUSY,TRUE,NULL);

	switch (who) {
	case 1:
		xv_set(CurrentContext->video,VIDEO_HUE,value,NULL);
		break;
	case 2:
		xv_set(CurrentContext->video,VIDEO_BRIGHTNESS,value,NULL);
		break;
	case 3:
		xv_set(CurrentContext->video,VIDEO_CONTRAST,value,NULL);
		break;
	case 4:
		xv_set(CurrentContext->video,VIDEO_SATURATION,value,NULL);
		break;
	case 5:
		(void) xv_get(CurrentContext->video,VIDEO_FACTORY_RESET);
		what = (int) xv_get(CurrentContext->video,VIDEO_HUE);
		xv_set(HueItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(CurrentContext->video,VIDEO_BRIGHTNESS);
		xv_set(BrightnessItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(CurrentContext->video,VIDEO_CONTRAST);
		xv_set(ContrastItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(CurrentContext->video,VIDEO_SATURATION);
		xv_set(SaturationItem,PANEL_VALUE,what,NULL);
		break;
	case 6:
		who = (int) xv_get(CurrentContext->video,VIDEO_CHANNEL);
		switch (who) {
		case VIDEO_IN_ONE:
			xv_set(CurrentContext->video,VIDEO_USER0,TRUE,NULL);
			break;
		case VIDEO_IN_TWO:
			xv_set(CurrentContext->video,VIDEO_USER1,TRUE,NULL);
			break;
		}
		break;
	case 7:
		who = (int) xv_get(CurrentContext->video,VIDEO_CHANNEL);
		switch (who) {
		case VIDEO_IN_ONE:
			(void) xv_get(CurrentContext->video,VIDEO_USER0);
			break;
		case VIDEO_IN_TWO:
			(void) xv_get(CurrentContext->video,VIDEO_USER1);
			break;
		}
		what = (int) xv_get(CurrentContext->video,VIDEO_HUE);
		xv_set(HueItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(CurrentContext->video,VIDEO_BRIGHTNESS);
		xv_set(BrightnessItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(CurrentContext->video,VIDEO_CONTRAST);
		xv_set(ContrastItem,PANEL_VALUE,what,NULL);
		what = (int) xv_get(CurrentContext->video,VIDEO_SATURATION);
		xv_set(SaturationItem,PANEL_VALUE,what,NULL);
		break;
	}
	xv_set(parent,FRAME_BUSY,FALSE,NULL);
}

static void MoveOutput()
{
	XGCValues gcv;
	GC gc;
	Xv_opaque root;
	Event event;
	Inputmask im;
	int x,y,w,h,ox,oy;
	Rect rect,bounds;
	int who,scale;

	xv_set(OutputFrame,
		FRAME_LEFT_FOOTER,	"SELECT=move/ADJUST=adjust/MENU=cancel",
		NULL);
	XSync(VAADisplay,False);

	scale = (int) xv_get(VideoOutput,VIDEO_OUT_SCALE);

	notify_dispatch();
	XFlush(VAADisplay);

	win_setinputcodebit(&im, MS_LEFT);
	win_setinputcodebit(&im, MS_MIDDLE);
	win_setinputcodebit(&im, MS_RIGHT);
	win_setinputcodebit(&im, LOC_DRAG);
	im.im_flags = im.im_flags | IM_NEGEVENT;

	FS = (Fullscreen) xv_create(VAAFrame,FULLSCREEN,
		WIN_CONSUME_EVENTS,		WIN_MOUSE_BUTTONS,LOC_DRAG,NULL,
		NULL);

	root = (Xv_opaque) RootWindow(VAADisplay,VAAScreen);

	gcv.function			= GXinvert;
	gcv.line_width			= 2;
	gcv.subwindow_mode		= IncludeInferiors;
	gc	= XCreateGC(VAADisplay,root,GCFunction | GCLineWidth | GCSubwindowMode,&gcv);

	frame_get_rect(VAAFrame,&rect);
	bounds.r_top   = 0;    bounds.r_left   = 0;
	bounds.r_width = 1152; bounds.r_height = 900;

/* Get output boundaries and display */

	x = (int) xv_get(VideoOutput,VIDEO_OUT_X);
	y = (int) xv_get(VideoOutput,VIDEO_OUT_Y);
	w = (int) xv_get(VideoOutput,VIDEO_OUT_WIDTH);
	h = (int) xv_get(VideoOutput,VIDEO_OUT_HEIGHT);

#ifdef SOLID
	XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
	XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif
	who = TRUE;
	event_init(&event);
	while (xv_input_readevent(VAAFrame,&event,TRUE,TRUE,&im) != -1) {
		if (event_is_down(&event) && (event_id(&event) == MS_MIDDLE)) {
			if (scale) {
				ox = event_x(&event) + rect.r_left;
				if (ox < (x + w / 2)) who = 0; else who = 2;
				oy = event_y(&event) + rect.r_top;
				if (oy > (y + h / 2)) who++;
			}
			else {
				xv_set(OutputFrame,
					FRAME_LEFT_FOOTER,	"Can't adjust FIXED region",
					NULL);
				break;
			}
		}
		else if (event_is_down(&event) && (event_id(&event) == MS_LEFT)) {
			who = 4;
			ox = event_x(&event);
			oy = event_y(&event);
		}
		else if (event_id(&event) == LOC_DRAG) {

/* Erase old box */

#ifdef SOLID
			XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
			XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif

/* Adjust accordingly */

			switch (who) {
			case 0:
				ox = event_x(&event) + rect.r_left;
				w -= ox - x; if (w < 40) w = 40;
				oy = event_y(&event) + rect.r_top;
				h -= oy - y; if (h < 30) h = 30;
				if (ox < bounds.r_left)  ox = bounds.r_left;
				if (oy < bounds.r_top)   oy = bounds.r_top;
				xv_set(VideoOutput,
					VIDEO_OUT_X,		ox,
					VIDEO_OUT_Y,		oy,
					VIDEO_OUT_WIDTH,	w,
					VIDEO_OUT_HEIGHT,	h,
					NULL);
				break;
			case 1:
				ox = event_x(&event) + rect.r_left;
				w -= ox - x; if (w < 40) w = 40;
				h  = event_y(&event) + rect.r_top - y;
				if (h < 30) h = 30;
				if (ox < bounds.r_left)  ox = bounds.r_left;
				xv_set(VideoOutput,
					VIDEO_OUT_X,		ox,
					VIDEO_OUT_WIDTH,	w,
					VIDEO_OUT_HEIGHT,	h,
					NULL);
				break;
			case 2:
				w  = event_x(&event) + rect.r_left - x;
				if (w < 40) w = 40;
				oy = event_y(&event) + rect.r_top;
				h -= oy - y; if (h < 30) h = 30;
				if (oy < bounds.r_top)   oy = bounds.r_top;
				xv_set(VideoOutput,
					VIDEO_OUT_Y,		oy,
					VIDEO_OUT_WIDTH,	w,
					VIDEO_OUT_HEIGHT,	h,
					NULL);
				break;
			case 3:
				w = event_x(&event) + rect.r_left - x;
				if (w < 40) w = 40;
				h = event_y(&event) + rect.r_top - y;
				if (h < 30) h = 30;
				xv_set(VideoOutput,VIDEO_OUT_WIDTH,w,VIDEO_OUT_HEIGHT,h,NULL);
				break;
			case 4:
				x += event_x(&event) - ox;
				y += event_y(&event) - oy;
				if (x < bounds.r_left) x = bounds.r_left;
				if (y < bounds.r_top)  y = bounds.r_top;
				if ((x + w) > bounds.r_width)  x = bounds.r_width  - w;
				if ((y + h) > bounds.r_height) y = bounds.r_height - h;
				xv_set(VideoOutput,VIDEO_OUT_X,x,VIDEO_OUT_Y,y,NULL);
				ox = event_x(&event); oy = event_y(&event);
				break;
			}

/* Redraw bounding area */

			x = (int) xv_get(VideoOutput,VIDEO_OUT_X);
			y = (int) xv_get(VideoOutput,VIDEO_OUT_Y);
			w = (int) xv_get(VideoOutput,VIDEO_OUT_WIDTH);
			h = (int) xv_get(VideoOutput,VIDEO_OUT_HEIGHT);
#ifdef SOLID
			XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
			XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif
		}
		else if (event_is_up(&event)) break;
		else if (event_id(&event) == MS_RIGHT) {
			xv_set(OutputFrame,
				FRAME_LEFT_FOOTER,	"Output Selection canceled",
				NULL);
			break;
		}
	}

/* Erase bounding box */

#ifdef SOLID
			XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
			XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif
	XFreeGC(VAADisplay,gc);

/* Destroy the fullscreen grab */

	xv_destroy(FS);
}

static void XFindWindow()
{
	XEvent event;
	Cursor cursor;
	Window window;
	Window root;
	XGCValues gcv;
	GC gc;
	int status;
	int x,y,w,h,ox,oy;

	xv_set(OutputFrame,
		FRAME_LEFT_FOOTER,	"SELECT - Select window   MENU - cancel",
		NULL);
	XSync(VAADisplay,False);

	root   = RootWindow(VAADisplay,VAAScreen);

	gcv.function			= GXinvert;
	gcv.line_width			= 2;
	gcv.subwindow_mode		= IncludeInferiors;
	gc	= XCreateGC(VAADisplay,root,GCFunction | GCLineWidth | GCSubwindowMode,&gcv);

/* Get output boundaries and display */

	x = (int) xv_get(VideoOutput,VIDEO_OUT_X);
	y = (int) xv_get(VideoOutput,VIDEO_OUT_Y);
	w = (int) xv_get(VideoOutput,VIDEO_OUT_WIDTH);
	h = (int) xv_get(VideoOutput,VIDEO_OUT_HEIGHT);

#ifdef SOLID
	XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
	XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif

	cursor = XCreateFontCursor(VAADisplay,XC_crosshair);
	status = XGrabPointer(VAADisplay,root,False,ButtonPressMask | ButtonReleaseMask,
				GrabModeSync,GrabModeAsync,root,cursor,CurrentTime);
	if (status != GrabSuccess) {
		ShowNote("Unable to grab the mouse, giving up!",NULL);
		return;
	}
	window = None;
	do {
		XAllowEvents(VAADisplay,SyncPointer,CurrentTime);
		XWindowEvent(VAADisplay,root,ButtonPressMask | ButtonReleaseMask,&event);
		switch (event.type) {
		case ButtonPress:
			if (event.xbutton.button == 3) {
				XUngrabPointer(VAADisplay,CurrentTime);
				xv_set(OutputFrame,
					FRAME_LEFT_FOOTER,	"Window selection canceled",
					NULL);
				return;
			}
			else if (window == None) {
				window=event.xbutton.subwindow;
				if (window == None) window=root;
			}
			break;
		default:
			break;
		}
	}
	while (window == None);

/* Erase old rectangle */

#ifdef SOLID
	XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
	XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif

	XGetGeometry(VAADisplay,window,&root,&x,&y,&w,&h,&ox,&oy);
	XQueryTree(VAADisplay,window,&root,&window,&ox,&oy);
	XFree(ox);
	XTranslateCoordinates(VAADisplay,window,root,x,y,&ox,&oy,&window);

	xv_set(VideoOutput,
		VIDEO_OUT_X,		ox,
		VIDEO_OUT_Y,		oy,
		VIDEO_OUT_WIDTH,	w,
		VIDEO_OUT_HEIGHT,	h,
		NULL);

/* Get output boundaries and display */

	x = (int) xv_get(VideoOutput,VIDEO_OUT_X);
	y = (int) xv_get(VideoOutput,VIDEO_OUT_Y);
	w = (int) xv_get(VideoOutput,VIDEO_OUT_WIDTH);
	h = (int) xv_get(VideoOutput,VIDEO_OUT_HEIGHT);

/* Grab the server while we highlight the screen */

	XGrabServer(VAADisplay);

#ifdef SOLID
	XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
	XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif

	XAllowEvents(VAADisplay,SyncPointer,CurrentTime);
	XWindowEvent(VAADisplay,root,ButtonPressMask | ButtonReleaseMask,&event);

#ifdef SOLID
	XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
	XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif

	XUngrabServer(VAADisplay);
	XUngrabPointer(VAADisplay,CurrentTime);
	XFreeGC(VAADisplay,gc);
	XFreeCursor(VAADisplay,cursor);
}

static void ShowOutput()
{
	XGCValues gcv;
	GC gc;
	Xv_opaque root;
	Event event;
	Inputmask im;
	int x,y,w,h,ox,oy;

	win_setinputcodebit(&im, MS_LEFT);
	win_setinputcodebit(&im, MS_MIDDLE);
	win_setinputcodebit(&im, MS_RIGHT);
	im.im_flags = im.im_flags | IM_NEGEVENT;

	FS = (Fullscreen) xv_create(VAAFrame,FULLSCREEN,
		WIN_CONSUME_EVENTS,		WIN_MOUSE_BUTTONS,NULL,
		NULL);

	root = (Xv_opaque) RootWindow(VAADisplay,VAAScreen);

	gcv.function			= GXinvert;
	gcv.line_width			= 2;
	gcv.subwindow_mode		= IncludeInferiors;
	gc	= XCreateGC(VAADisplay,root,GCFunction | GCLineWidth | GCSubwindowMode,&gcv);

/* Get output boundaries and display */

	x = (int) xv_get(VideoOutput,VIDEO_OUT_X);
	y = (int) xv_get(VideoOutput,VIDEO_OUT_Y);
	w = (int) xv_get(VideoOutput,VIDEO_OUT_WIDTH);
	h = (int) xv_get(VideoOutput,VIDEO_OUT_HEIGHT);
#ifdef SOLID
	XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
	XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif
	event_init(&event);
	xv_input_readevent(VAAFrame,&event,TRUE,TRUE,&im);

/* Erase bounding box */
#ifdef SOLID
	XFillRectangle(VAADisplay,root,gc,x,y,w,h);
#else
	XDrawRectangle(VAADisplay,root,gc,x,y,w,h);
#endif
	XFreeGC(VAADisplay,gc);
/* Destroy the fullscreen grab */

	xv_destroy(FS);
}

void OutputPanelProc(item, value)
Panel_item item;
int value;
{
	Menu menu;
	Panel parent;
	int who,what;

	who	   = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	parent = (Panel) xv_get(item,PANEL_PARENT_PANEL);

	xv_set(parent,FRAME_BUSY,TRUE,NULL);

	switch (who) {
	case 1:
		xv_set(VideoOutput,VIDEO_OUTPUT,value,NULL);
		break;
	case 2:
		if (SelectMode) MoveOutput(); else XFindWindow();
		break;
	case 3:
		switch (value) {
		case 0:
			xv_set(VideoOutput,VIDEO_OUT_ASPECT,TRUE,NULL);
			xv_set(VideoOutput,VIDEO_OUT_SCALE,TRUE,NULL);
			break;
		case 1:
			xv_set(VideoOutput,VIDEO_OUT_ASPECT,TRUE,NULL);
			xv_set(VideoOutput,VIDEO_OUT_SCALE,FALSE,NULL);
/* FIXUP: Should be getting the signal width and height from the
   VideoOutput object itself
*/
			xv_set(VideoOutput,
				VIDEO_OUT_WIDTH,		640,
				VIDEO_OUT_HEIGHT,		480,
				NULL);
			break;
		case 2:
			xv_set(VideoOutput,VIDEO_OUT_ASPECT,FALSE,NULL);
			xv_set(VideoOutput,VIDEO_OUT_SCALE,TRUE,NULL);
			break;
		}
		break;
	case 4:
		SelectMode = value;
		break;
	case 5:
		switch (value) {
		case 0:
			what = xv_set(VideoOutput,VIDEO_OUT_FORMAT,VIDEO_COMPOSITE,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 1:
			what = xv_set(VideoOutput,VIDEO_OUT_FORMAT,VIDEO_SUPER_VHS,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 2:
			what = xv_set(VideoOutput,VIDEO_OUT_FORMAT,VIDEO_YUV,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 3:
			what = xv_set(VideoOutput,VIDEO_OUT_FORMAT,VIDEO_RGB,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_FORMAT) {
					ShowAlert("This format is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		}
		break;
	case 6:
		switch (value) {
		case 0:
			what = xv_set(VideoOutput,VIDEO_OUT_STANDARD,VIDEO_NTSC,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_STANDARD) {
					ShowAlert("This video standard is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 1:
			what = xv_set(VideoOutput,VIDEO_OUT_STANDARD,VIDEO_PAL,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_STANDARD) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		case 2:
			what = xv_set(VideoOutput,VIDEO_OUT_STANDARD,VIDEO_SECAM,NULL);
			if (what == XV_ERROR) {
				if (Xv_VideoStatus == VIDEO_BAD_STANDARD) {
					ShowAlert("This video input type is not supported",NULL);
				}
				xv_set(item,PANEL_VALUE,xv_get(item,XV_KEY_DATA,ITEM_DATA),NULL);
				xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
			}
			else xv_set(item,XV_KEY_DATA,ITEM_DATA,value,NULL);
			break;
		}
		break;
	}
	xv_set(parent,FRAME_BUSY,FALSE,NULL);
}

/* end of video.c */

